﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace CalcCluster
{
    public partial class Form1 : Form, IprintResult
    {
        String sPath = System.Environment.CurrentDirectory;
        SearchThread st=null;
        Thread th = null;
        Cluster[] clusters;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            editor.Dock = DockStyle.Fill;
            tableLayoutPanel1.Dock = DockStyle.Fill;

            startDirectory.Text = System.Environment.CurrentDirectory;
            clusters = new Cluster[8];
            clusters[0] = new Cluster(512);
            clusters[1] = new Cluster(1024);
            clusters[2] = new Cluster(2048);
            clusters[3] = new Cluster(4096);
            clusters[4] = new Cluster(8192);
            clusters[5] = new Cluster(16384);
            clusters[6] = new Cluster(32768);
            clusters[7] = new Cluster(65536);
            labelActualDir.Visible = false;
            aktualDir.Visible = false;
            bnStopThread.Visible = false;
        }

        private void bnSelectDir_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog objDialog = new FolderBrowserDialog();
            objDialog.Description = "Startverzueichnis";
            objDialog.SelectedPath = sPath;   
            DialogResult objResult = objDialog.ShowDialog(this);
            if (objResult == DialogResult.OK)
            {
                startDirectory.Text = objDialog.SelectedPath;
            }
        }

        private void mnPictures_Click(object sender, EventArgs e)
        {
            Calc(SearchThread.SearchModi.Pictures);
        }

        private void mnMediaFiles_Click(object sender, EventArgs e)
        {
            Calc(SearchThread.SearchModi.Media);
        }

        private void mnAllFiles_Click(object sender, EventArgs e)
        {
            Calc(SearchThread.SearchModi.AllFiles);
        }

        private void Calc(SearchThread.SearchModi modi)
        {
            editor.Text = "";
            if (Directory.Exists(startDirectory.Text))
            {
                labelActualDir.Visible = true;
                aktualDir.Visible = true;
                bnStopThread.Visible = true;
                st = new SearchThread(startDirectory.Text, clusters, modi, this);

                ThreadStart ts = new ThreadStart(st.run);
                th = new Thread(ts);
                th.Start();
                //th.Join();
            }

        }  // bnCalc

        delegate void printStatusCallback(string dir);

        public void printStatus(String dir)
        {
            if (this.aktualDir.InvokeRequired)
            {
                printStatusCallback d = new printStatusCallback(printStatus);
                this.Invoke(d, new object[] { dir });
            }
            else
            {
                this.aktualDir.Text = dir;
                if (dir.Equals(""))
                {
                    labelActualDir.Visible = false;
                    aktualDir.Visible = false;
                }
            }
        }


        delegate void AddTextCallback(string text);
        public void AddText(String text)
        {
            if (this.editor.InvokeRequired)
            {
                AddTextCallback d = new AddTextCallback(AddText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.editor.Text += text;
            }
        }  // SetAddText


        delegate void SetTextCallback(string text);
        public void SetText(String text)
        {
            if (this.editor.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.editor.Text = text;
            }
        }  // SetAddText

        public void printResult()
        {
            long summe;
            long anzFiles;
            summe = st.summe;
            anzFiles = st.anzFiles;
            String sEditor = "";

            sEditor = "Cluster-Analyse\r\n";
            sEditor += ("Startverzeichnis: " + startDirectory.Text + "\r\n\r\n");
            sEditor +=("Gesamtsumme: " + summe + "\r\n\r\n");
            sEditor +=("Anzahl der Dateien: " + anzFiles + "\r\n\r\n");
            foreach (Cluster c in clusters)
            {
                sEditor +=("Clustergröße: " + c.size + " Byte"+  "   anz: " + c.anz 
                    + "    Gesamtgröße: " + Basis.best_KB(c.size * c.anz)
                    + "    Diff: " + Basis.best_KB(c.size * c.anz - summe) + "\r\n\r\n");
            }
            SetText(sEditor);
            printStatus("");
        }

       

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (th!=null  && th.IsAlive)
            {
                th.Abort();
            }
        }

        private void mnClose_Click(object sender, EventArgs e)
        {
            Close();
        }


        private void mnProginfo_Click(object sender, EventArgs e)
        {
            FProgInfo dialog = new FProgInfo();
            dialog.ShowDialog();
        }

        private void bnStopThread_Click(object sender, EventArgs e)
        {
            if (th != null && th.IsAlive)
            {
                th.Abort();
                th = null;
                labelActualDir.Visible = false;
                aktualDir.Visible = false;
                bnStopThread.Visible = false;
            }

        }


    }
}
