
import javax.swing.*; // JPanel Timer
import java.awt.*;
import java.awt.event.*;
import java.util.Stack;
import java.util.ArrayList;

public class ScheibenCanvas extends JPanel implements ActionListener {

	private ArrayList<Verschiebung> listMoving = new ArrayList<Verschiebung>();

	private javax.swing.Timer timer;
	private int timerintervall = 1000;

	private Kasten k1 = null;
	private Kasten k2 = null;
	private Kasten k3 = null;
	private int index = 0;
	private int anzScheiben = 0;
	private Font font = new Font("Arial", Font.BOLD, 24);

	public boolean timerAbbruch = false;

	public ScheibenCanvas() {
		setDoubleBuffered(true);
	}


	
	public void startHanoi(int n, int timerintervall) {
		k1 = k2 = k3 = null;
		drawInit();
		this.timerintervall = timerintervall;
		this.anzScheiben = n;
		Graphics g = this.getGraphics();		
		int w = k1.getWidth();
		int diff = w / (n + 1);
		int h = k1.getHeight() / ((n) << 1);
		for (int i = n; i > 0; i--) {
			k1.addEbene(new Scheibe(w, h));
			w = w - diff;
		}
		paint(g);
		inStack();
	}
	
	public void drawInit() {
		int w = getWidth() / 16;
		int y = getHeight() - 80;
		System.out.println("w: " + w);
		if (w > 0 && y > 0) {
			if (k1 == null)
				k1 = new Kasten("A", w * 3, y, w << 2, getHeight() - 200, Color.RED, new Color(150, 0, 0));
			else
				k1.setBounds(w * 3, y, w << 2, getHeight() - 200);
			if (k2 == null)
				k2 = new Kasten("B", w * 3 + 5 * w, y, w << 2, getHeight() - 200, Color.GREEN, new Color(0, 150, 0));
			else
				k2.setBounds(w * 3 + 5 * w, y, w << 2, getHeight() - 200);
			if (k3 == null)
				k3 = new Kasten("C", w * 3 + 10 * w, y, w << 2, getHeight() - 200, Color.BLUE, new Color(0, 0, 150));
			else
				k3.setBounds(w * 3 + 10 * w, y, w << 2, getHeight() - 200);
		}
	} // drawInit	
	
	public void startTimer() {
		// in der Methode "drawMove" wird 3x timerintervall aufgerufen
		// bei zu kleiner Wartezeit gibt es ein EmptyStackException, 
		// da die Bewegung noch nicht abgeschlossen war 
		timer = new Timer(timerintervall, this);
		timer.start();
	}	
	
	private void inStack() {
		listMoving.clear();   
		bewegeInStack(anzScheiben, k1, k2, k3);
		index=0;
		int n = listMoving.size();
		System.out.println("n: "+n);
	}



	// eintragen der Befehle in einen Stack
	// per Timer oder Click abarbeiten
	void bewegeInStack(int i, Kasten a, Kasten b, Kasten c) {
		if (i > 0) {
			bewegeInStack(i - 1, a, c, b);
			// verschiebe oberste Scheibe von a nach c;
			index++;
			System.out.println(index + ":  verschiebe oberste Ebene von " + a.getBez() + "  nach " + c.getBez());
			//drawMove(a, c);  // alter Befehl
			listMoving.add( new Verschiebung(a, c));			
			bewegeInStack(i - 1, b, a, c);
		}
	} // bewegeInStack

	public void nextStep() {
		if (index<listMoving.size()) {
			Verschiebung verschiebung = listMoving.get(index);
			index++;
			drawMove(verschiebung.quelle, verschiebung.ziel);
		}
		else {
			if (timer!=null) {
				timer.stop();
				timer=null;
			}
		}
	}

	
	
	@Override
	public void actionPerformed(ActionEvent arg0) {
		nextStep();
	}	
	public void stopTimer() {
		if (timer != null) {
			timer.stop();
			timer=null;
			k1 = null;
			k2 = null;
			k3 = null;
		}
	}	
	
//	void bewege(int i, Kasten a, Kasten b, Kasten c) {
//		if (i > 0) {
//			Graphics g = this.getGraphics();
//			bewege(i - 1, a, c, b);
//			// verschiebe oberste Scheibe von a nach c;
//			index++;
//			System.out.println(index + ":  verschiebe oberste Ebene von " + a.getBez() + "  nach " + c.getBez());
//			drawMove(a, c);
//			paint(g);
//			warten();
//			bewege(i - 1, b, a, c);
//		}
//	}

	private void drawMove(Kasten a, Kasten b) {
		Graphics g = this.getGraphics();		
		Scheibe ebene = a.removeEbene();
		//warten();
		paint(g);
		ebene.draw(g, a.getColor1(), a.getX(), 100);
		warten();

		paint(g); // draw
		ebene.draw(g, b.getColor1(), b.getX(), 100);
		warten();
		
		b.addEbene(ebene);
		paint(g); // draw
	}

	

	public void stop() {
	}

	private void warten() {
		try {
			Thread.sleep(timerintervall>>1);
		} catch (InterruptedException ex) {
			// Logger.getLogger(Pyramide_Grf.class.getName()).log(Level.SEVERE,
			// null, ex);
		}
	}

	@Override
	public void paint(Graphics g) {
		Graphics2D g2d=(Graphics2D)g;
		g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON );
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());
		g.setColor(Color.BLUE);
		g.setFont(font);
		String bez = "Die Trme von Hanoi";
		FontMetrics fontMetrics = g.getFontMetrics(g.getFont());
		int breite = fontMetrics.stringWidth(bez);
		g.drawString(bez, (getWidth() >> 1) - (breite >> 1), 40);
		g.drawString("Schritt: "+Integer.toString(index), 10, 40);
		drawInit(); // die Grenzen werden immer berprft
		k1.draw(g);
		k2.draw(g);
		k3.draw(g);
	} // paint

	

}
